@@ -1,4 +1,4 @@
-from .fhir_query import FHIRQuery
+from .fhir_query import FHIRQuerySync, FHIRQueryAsync
 from .query_response import QueryResponse
 from .query_parameters import QueryOperators, FHIRQueryParameters, FieldParameter, IncludeParameter, \
     ReverseChainParameter
