@@ -1376,7 +1376,7 @@ def test_query_json(server):
 def test_query_first(server):
     query = server.query("Patient")
     result = query.first()
-
+    print(result)
     assert len(result.resources) == 1
     assert isinstance(result.resources[0], Patient)
 
@@ -1384,26 +1384,24 @@ def test_query_first(server):
 def test_query_limit(server):
     query = server.query("Patient")
     query._count = 30
-    result = query.limit(100)
+    result = query.limit(100, page_callback=lambda x: print(len(x)))
     assert len(result.resources) == 100
 
+    query = server.query("Patient", output_format="xml").all(count=50)
+
+
 
 def test_query_with_callback(server):
-    def callback1(resources):
-        print(len(resources))
+    print("callback1")
+    result = server.query("Patient").all(page_callback=lambda: print("callback"), count=50)
 
-    server.query("Patient").all(page_callback=callback1, count=50)
-
-    def callback2():
-        print("callback2")
-
-    server.query("Patient").all(page_callback=callback2, count=200)
+    print("callback2")
+    result = server.query("Patient").all(page_callback=lambda x: print(len(x)), count=20)
 
+    print("callback3")
     with pytest.raises(ValueError):
-        def callback3(a, b):
-            print("callback3")
 
-        server.query("Patient").all(page_callback=callback3, count=200)
+        server.query("Patient").all(page_callback=lambda x,y: print(x, y), count=20)
 
 
 def test_query_response_resources(server):
@@ -1665,10 +1663,10 @@ def test_query_response_save(server):
     query_resource = "Condition"
     query = server.query(query_resource, output_format="xml")
     response = query.limit(100)
-    response.save(xml_file, format="xml")
+    response.save(xml_file, output_format="xml")
 
     with pytest.raises(NotImplementedError):
-        response.save(xml_file, format="json")
+        response.save(xml_file, output_format="json")
 
     assert os.path.isfile(xml_file)
     assert xmltodict.parse(open(xml_file, "r").read()) == xmltodict.parse(response.response)
@@ -1677,7 +1675,7 @@ def test_query_response_save(server):
 
     # invalid format
     with pytest.raises(ValueError):
-        response.save(xml_file, format="invalid")
+        response.save(xml_file, output_format="invalid")
 
     # test with included resources and json
     query_resource = "Condition"
@@ -1691,7 +1689,7 @@ def test_query_response_save(server):
     query = query.include(resource=query_resource, reference_param=search_param)
     response = query.all()
 
-    response.save(json_file, format="json")
+    response.save(json_file, output_format="json")
 
     assert os.path.isfile(json_file)
     assert json.loads(open(json_file).read())
@@ -1700,7 +1698,7 @@ def test_query_response_save(server):
 
     # save to csv
     patient_response = server.query("Patient").all()
-    patient_response.save(csv_file, format="csv")
+    patient_response.save(csv_file, output_format="csv")
 
     assert os.path.isfile(csv_file)
     assert pd.read_csv(csv_file).shape[0] > 0
@@ -1708,7 +1706,7 @@ def test_query_response_save(server):
     # os.remove(csv_file)
 
     # save to csv with included resource
-    response.save(csv_file, format="csv")
+    response.save(csv_file, output_format="csv")
 
     assert os.path.isfile(csv_file)
     assert pd.read_csv(csv_file).shape[0] > 0
@@ -1735,9 +1733,27 @@ def test_query_response_to_dfs(server):
     response = query.all()
 
     with pytest.raises(ValueError):
-        response.to_dfs(format="invalid")
+        response.to_dfs(df_format="invalid")
 
     dfs = response.to_dfs()
 
     assert len(dfs) > 1
     assert dfs[0].shape[0] > 0
+
+
+@pytest.mark.asyncio
+async def test_query_async(server):
+    query = server.query_async("Patient")
+    response = await query.all()
+    print(response)
+
+    with pytest.raises(ValueError):
+        query = server.query_async("Patient", output_format="invalid")
+        response = await query.all()
+        print(response)
+
+@pytest.mark.asyncio
+async def test_query_async_xml(server):
+    query = server.query_async("Patient", output_format="xml")
+    response = await query.all()
+    print(response)
\ No newline at end of file
